@@ -8,6 +8,7 @@ import tinychain.scalar
 
 from tinychain.collection import Column
 from tinychain.collection import btree, table, tensor
+from tinychain.context import print_json, to_json, Context
 from tinychain.decorators import closure, delete, differentiable, get, post, put
 from tinychain.graph import Edge, Graph
 from tinychain.math import linalg
@@ -15,7 +16,8 @@ from tinychain.reflect import Meta
 from tinychain.generic import Map, Tuple
 from tinychain.interface import Interface, Equality, Order
 from tinychain.scalar.ref import After, Case, If, While  # TODO: replace with helper methods
+from tinychain.scalar.ref import form_of, get_ref, is_literal, is_conditional, is_none, is_op, is_op_ref, is_ref, same_as
 from tinychain.scalar.value import EmailAddress, Id, String, Value
 from tinychain.scalar.number import Number, Bool, Complex, C32, C64, Float, F32, F64, Int, I16, I32, I64, UInt, U8, U16, U32, U64
 from tinychain.state import Class, Instance, State, Stream
-from tinychain.util import form_of, print_json, to_json, uri, Context, URI
+from tinychain.uri import uri, URI
