@@ -7,7 +7,8 @@ import urllib.parse
 
 from .error import *
 from .scalar.value import Nil
-from .util import to_json, uri, URI
+from .uri import uri, URI
+from .context import to_json
 
 
 DEFAULT_PORT = 8702
