@@ -1,4 +1,4 @@
 """:class:`Interface` and :class:`Operator` definitions for common mathematical operations"""
 
 from .base import product, sum
-from .operator import Operator
+from .operator import constant, derivative_of, gradients, is_constant, Gradients, Operator
