@@ -1,29 +1,67 @@
 import inspect
 import logging
+import typing
 
 from .. import error
 from ..app import Dynamic, Model, ModelRef
 from ..collection.tensor import Dense, Tensor
 from ..decorators import post
 from ..generic import Map, Tuple
-from ..math.operator import derivative_of, Operator
+from ..math.interface import Numeric
+from ..math.operator import constant, derivative_of, is_constant, simplify
 from ..scalar.number import F32, F64, UInt
-from ..scalar.ref import After, is_op_ref
-from ..util import form_of, hex_id
+from ..scalar.ref import form_of, hex_id, After, If
+from ..scalar.value import Id
+from ..state import State
 
 from .variable import Variable
 from . import LIB_URI
 
 
+# this helper class handles the case of both a Number and a Tensor as a gradient
+class _Gradient(State, Numeric):
+    pass
+
+
+_Gradients = typing.Dict[Id, _Gradient]
+
+
 class Optimizer(Model):
     __uri__ = LIB_URI.append("Optimizer")
 
     @post
-    def train(self, inputs):
+    def train(self, i, inputs):
         return error.NotImplemented(f"{self.__class__.__name__}.train")
 
 
-class GradientDescent(Optimizer, Dynamic):
+class _Optimizer(Optimizer, Dynamic):
+    @post
+    def gradients(self, cxt, inputs: Tensor) -> _Gradients:
+        logging.debug("Optimizer constructing gradient calculations...")
+
+        trainable_vars = trainable(self.ml_model)
+        logging.debug("discovered trainable variables...")
+
+        outputs = self.ml_model.eval(inputs)
+        logging.debug("constructed model evaluation")
+
+        d_loss = derivative_of(self._cost(inputs, outputs))
+        logging.debug("constructed derivative of loss")
+        cxt.d_loss = constant(d_loss.copy() if isinstance(d_loss, Tensor) else d_loss)
+        assert is_constant(cxt.d_loss)
+
+        grads = {var_id: simplify(derivative_of(outputs, cxt.d_loss, var)) for var_id, var in trainable_vars.items()}
+        logging.debug("constructed gradients")
+
+        if not grads:
+            raise ValueError(f"model output {outputs} has no gradients")
+
+        return {
+            var_id: If(grad.shape[1:] == trainable_vars[var_id].shape, grad.sum(0), grad.sum())
+            for var_id, grad in grads.items()}
+
+
+class GradientDescent(_Optimizer):
     """A simple gradient descent optimizer with a configurable learning rate."""
 
     def __init__(self, ml_model, cost, learning_rate=0.001):
@@ -38,30 +76,18 @@ class GradientDescent(Optimizer, Dynamic):
         Dynamic.__init__(self)
 
     @post
-    def train(self, i: UInt, inputs: Tensor) -> Tensor:
-        outputs = self.ml_model.eval(inputs)
-        operator = form_of(outputs)
-
-        if not isinstance(operator, Operator):
-            raise ValueError(f"Optimizer can only train a differentiable Operator, not {form_of(outputs)}")
-
-        loss = self._cost(inputs, outputs)
-        d_loss = derivative_of(loss).copy()
-        gradients = operator.gradients(d_loss)
-
-        validate(self.ml_model, self._model_name, operator, gradients)
-
-        variables = trainable(self.ml_model)
+    def train(self, txn, i: UInt, inputs: Tensor) -> Tensor:
+        grads = self.gradients(inputs)
 
         writes = []
-        for var_id, delta in gradients.items():
-            var = variables[var_id]
-            writes.append(var.update(delta * self._lr))
+        for var_id, var in trainable(self.ml_model).items():
+            delta = grads[var_id]
+            writes.append(var.update(self._lr * delta))
 
-        return After(writes, loss)
+        return writes
 
 
-class Adam(Optimizer, Dynamic):
+class Adam(_Optimizer):
     """
     Adam optimizer, an adaptive learning rate optimization algorithm designed to handle sparse gradients and noisy data.
 
@@ -86,7 +112,7 @@ class Adam(Optimizer, Dynamic):
         for name, var in namespace(ml_model, self._model_name).items():
             shape = form_of(var.shape)
             if not isinstance(shape, (list, tuple)):
-                raise ValueError(f"the shape of a Variable must be defined at compile time (found {shape})")
+                raise ValueError(f"the shape of Variable {name} must be defined at compile time (found {shape})")
 
             self.m[name] = Dense.constant(shape, 0)
             self.v[name] = Dense.constant(shape, 0)
@@ -94,96 +120,72 @@ class Adam(Optimizer, Dynamic):
         Dynamic.__init__(self)
 
     @post
-    def train(self, i: UInt, inputs: Tensor) -> Tensor:
-        outputs = self.ml_model.eval(inputs)
-        operator = form_of(outputs)
+    def train(self, txn, i: UInt, inputs: Tensor) -> Tensor:
+        grads = self.gradients(inputs)
 
-        if not isinstance(operator, Operator):
-            raise ValueError(f"Optimizer can only train a differentiable Operator, not {operator}")
+        trainable_vars = trainable(self.ml_model)
+        var_names = namespace(self.ml_model, self._model_name)
+        var_names = {hex_id(var): name for name, var in var_names.items()}
+        grads = {var_names[var_id]: grads[var_id] for var_id in var_names.keys()}
 
-        loss = self._cost(inputs, outputs)
-        d_loss = derivative_of(loss).copy()
-        gradients = operator.gradients(d_loss)
-
-        vars, var_names = validate(self.ml_model, self._model_name, operator, gradients)
-        vars = {name: vars[var_id] for var_id, name in var_names.items()}
-
-        gradients = {var_names[var_id]: delta for var_id, delta in gradients.items()}
+        vars_by_name = {var_names[var_id]: trainable_vars[var_id] for var_id in var_names.keys()}
 
         update_m = {}
         for name in self.m:
-            grad = gradients[name]
+            grad = grads[name]
             update_m[name] = self.m[name] * self.beta1 * grad * (1. - self.beta1)
 
         update_v = {}
         for name in self.v:
-            grad = gradients[name]
+            grad = grads[name]
             update_v[name] = self.v[name] * self.beta2 + grad**2 * (1. - self.beta2)
 
-        update_v = {name: self.v[name] * self.beta2 + gradients[name]**2 * (1. - self.beta2) for name in self.v}
+        update_v = {name: self.v[name] * self.beta2 + grads[name]**2 * (1. - self.beta2) for name in self.v}
 
         a = self.lr * (1. - self.beta2**i)**0.5 / (1 - self.beta1**i)
-        update_model = {name: self.m[name] / (self.v[name]**0.5 + self.eps) * a for name in gradients}
+        update_model = {name: self.m[name] / (self.v[name]**0.5 + self.eps) * a for name in grads}
 
         updates = After([
             [self.m[name].write(new_value) for name, new_value in update_m.items()],
             [self.v[name].write(new_value) for name, new_value in update_v.items()],
-        ], [vars[name].update(delta) for name, delta in update_model.items()])
+        ], [vars_by_name[name].update(delta) for name, delta in update_model.items()])
 
-        return After(updates, loss)
+        return updates
 
 
-def validate(model, name, operator, gradients):
-    """Check that the :class:`Variables` of `model` are trainable using the given `operator`."""
+class _Queue(object):
+    def __init__(self, *nodes):
+        self._queue = []
 
-    ns = {hex_id(var): name for name, var in namespace(model, name).items()}
+        for node in nodes:
+            self.push(node)
 
-    assert isinstance(operator, Operator)
-    assert ns
+    def __bool__(self):
+        return bool(self._queue)
 
-    missing_vars = set(ns.keys())
-    vars = {}
-    visited = []
-    unvisited = [operator.subject, operator.args]
-    while unvisited:
-        node = unvisited.pop()
+    def __getitem__(self, key):
+        return self._queue[key]
 
-        if isinstance(form_of(node), Operator):
-            assert hex_id(node) not in missing_vars
+    def __repr__(self):
+        return str(self._queue)
 
-            node = form_of(node)
-            unvisited.append(node.subject)
-            unvisited.append(node.args)
-            visited.append(node)
-
-        elif isinstance(node, Variable):
-            if hex_id(node) in missing_vars:
-                logging.debug(f"found Variable {ns[hex_id(node)]}")
-            else:
-                raise RuntimeError(f"{operator} node {node} references unknown Variable {hex_id(node)} (known: {ns})")
-
-            vars[hex_id(node)] = node
-            missing_vars.remove(hex_id(node))
+    def push(self, node):
+        if node is None:
+            return
+        elif isinstance(node, (Map, Tuple)):
+            return self.push(form_of(node))
 
+        if isinstance(node, (list, tuple)):
+            for item in node:
+                self.push(item)
+        elif isinstance(node, dict):
+            for item in node.values():
+                self.push(item)
         else:
-            logging.debug(f"skipping non-trainable operator graph node {node}")
+            self._queue.append(node)
 
-    if missing_vars:
-        missing = set(ns[var_id] for var_id in missing_vars)
-        if visited:
-            raise RuntimeError(f"{name} operator graph disconnects Variables {missing} at {visited[-1]}")
-        else:
-            raise RuntimeError(f"{name} operator graph {operator} is not connected to its Variables {missing}")
-
-    missing_grads = set(ns[var_id] for var_id in set(ns.keys()) - set(gradients.keys()))
-    if missing_grads:
-        raise RuntimeError(f"optimizer has gradients for {set(gradients.keys())} but not {missing_grads}")
-
-    extra_grads = set(gradients.keys()) - set(ns.keys())
-    if extra_grads:
-        raise RuntimeError(f"optimizer found gradients {extra_grads} without corresponding Variables")
-
-    return vars, ns
+    def shift(self):
+        return self._queue.pop(0)
 
 
 def namespace(model, prefix):
