@@ -3,9 +3,10 @@ import inspect
 from ..scalar import Value
 from ..scalar import op, ref
 from ..state import State
-from ..util import form_of, to_json, uri, Context, URI
+from ..uri import uri, URI
+from ..context import to_json, Context
 
-from . import _get_rtype, is_none, parse_args, resolve_class
+from . import _get_rtype, parse_args, resolve_class
 
 
 EMPTY = inspect.Parameter.empty
@@ -29,7 +30,7 @@ class Method(object):
         return False
 
     def __json__(self):
-        return {str(uri(self)): to_json(form_of(self))}
+        return {str(uri(self)): to_json(ref.form_of(self))}
 
     def dtype(self):
         return self.__class__.__name__
@@ -49,10 +50,10 @@ class Get(Method):
         self.rtype = _get_rtype(self.form, State)
 
     def __call__(self, key=None):
-        return self.rtype(ref.Get(self.subject(), key))
+        return self.rtype(form=ref.Get(self.subject(), key))
 
     def __args__(self):
-        _, cxt = form_of(self)
+        _, cxt = ref.form_of(self)
         return [self.subject(), cxt]
 
     def __form__(self):
@@ -84,7 +85,7 @@ class Put(Method):
     def __init__(self, header, form, name):
         rtype = _get_rtype(form, None)
 
-        if not is_none(rtype):
+        if not ref.is_none(rtype):
             raise ValueError(f"PUT method must return None, not f{rtype}")
 
         Method.__init__(self, header, form, name)
@@ -93,7 +94,7 @@ class Put(Method):
         return ref.Put(self.subject(), key, value)
 
     def __args__(self):
-        _, _, cxt = form_of(self)
+        _, _, cxt = ref.form_of(self)
         return [self.subject(), cxt]
 
     def __form__(self):
@@ -161,10 +162,10 @@ class Post(Method):
             sig = sig[1:]
 
         params = parse_args(sig, *args, **kwargs)
-        return rtype(ref.Post(self.subject(), params))
+        return rtype(form=ref.Post(self.subject(), params))
 
     def __args__(self):
-        _, cxt = form_of(self)
+        _, cxt = ref.form_of(self)
         return [self.subject(), cxt]
 
     def __form__(self):
@@ -174,7 +175,15 @@ class Post(Method):
         kwargs = {}
         for name in list(sig.parameters.keys())[len(args):]:
             param = sig.parameters[name]
-            dtype = resolve_class(self.form, param.annotation, State)
+
+            dtype = State
+            if param.default is inspect.Parameter.empty:
+                if param.annotation:
+                    dtype = param.annotation
+            elif isinstance(param.default, State):
+                dtype = type(param.default)
+
+            dtype = resolve_class(self.form, dtype, State)
             kwargs[name] = dtype(form=URI(name))
 
         cxt._return = self.form(*args, **kwargs)
@@ -192,7 +201,7 @@ class Delete(Method):
     def __init__(self, header, form, name):
         rtype = _get_rtype(form, None)
 
-        if not is_none(rtype):
+        if not ref.is_none(rtype):
             raise ValueError(f"DELETE method must return None, not f{rtype}")
 
         Method.__init__(self, header, form, name)
@@ -201,7 +210,7 @@ class Delete(Method):
         return ref.Delete(self.subject(), key)
 
     def __args__(self):
-        _, cxt = form_of(self)
+        _, cxt = ref.form_of(self)
         return [self.subject(), cxt]
 
     def __form__(self):
@@ -210,7 +219,12 @@ class Delete(Method):
 
 class Operator(Post):
     def __call__(self, *args, **kwargs):
-        return self.form(self.header, *args, **kwargs)
+        result = self.form(self.header, *args, **kwargs)
+
+        if self.rtype:
+            return self.rtype(form=result)
+        else:
+            return result
 
 
 def _check_context_param(parameter):
