@@ -1,6 +1,7 @@
 from ..interface import Equality
 from ..state import State
-from ..util import form_of, to_json, uri
+from ..uri import uri
+from ..context import to_json
 
 
 class Scalar(State, Equality):
@@ -13,6 +14,7 @@ class Scalar(State, Equality):
     __uri__ = uri(State) + "/scalar"
 
     def __json__(self):
+        from .ref.helpers import form_of
         return to_json(form_of(self))
 
     def eq(self, other):
