@@ -4,9 +4,9 @@ import inspect
 
 from .base import _Base
 from .interface import Functional
-from .reflect import is_ref
-from .scalar.ref import Ref
-from .util import deanonymize, form_of, get_ref, hex_id, to_json, uri, URI
+from .scalar.ref import form_of, get_ref, hex_id, is_ref, Ref
+from .uri import uri, URI
+from .context import deanonymize, to_json
 
 
 class State(_Base):
@@ -23,24 +23,26 @@ class State(_Base):
             if not hasattr(self, "__form__") or self.__form__ is None:
                 raise ValueError(f"instance of {self.__class__.__name__} has no form")
         else:
+            while isinstance(form, State):
+                form = form_of(form)
+
             # make sure to capture the URI of the given form
             if isinstance(form, URI):
                 self.__uri__ = form
             elif is_ref(form) and hasattr(form, "__uri__"):
+                assert uri(form)
                 self.__uri__ = uri(form)
 
-            # then discard any intermediate type expectations
-            while form_of(form) is not form:
-                form = form_of(form)
-
             self.__form__ = form
 
+        assert not isinstance(form_of(self), State)
+
         _Base.__init__(self)
 
     def __json__(self):
         form = form_of(self)
 
-        if is_ref(form):
+        if uri(form) == uri(self):
             return to_json(form)
         else:
             return {str(uri(self)): [to_json(form)]}
@@ -48,10 +50,10 @@ class State(_Base):
     def __id__(self):
         return hex_id(form_of(self))
 
-    def __ns__(self, cxt):
+    def __ns__(self, cxt, name_hint):
         form = form_of(self)
 
-        deanonymize(form, cxt)
+        deanonymize(form, cxt, name_hint)
 
         if isinstance(self.__form__, URI):
             self.__uri__ = self.__form__
@@ -76,16 +78,7 @@ class State(_Base):
             raise NotImplementedError("dtype to cast into must be known at compile-time")
 
         from .scalar.ref import Get
-        return dtype(Get(dtype, self))
-
-    def copy(self):
-        """Create a new `State` by copying this one."""
-
-        return self._get("copy", rtype=self.__class__)
-
-    def dtype(self):
-        """Return the native base :class:`Class` of this `State`."""
-        return self._get("class", rtype=Class)
+        return dtype(form=Get(dtype, self))
 
     def hash(self):
         """Return the SHA256 hash of this `State` as an :class:`Id`."""
@@ -180,14 +173,25 @@ class StateRef(Ref):
         self.state = state
         self.__uri__ = URI(name)
 
+    def __repr__(self):
+        is_auto_assigned = False
+
+        address = str(uri(self)).split('_')[-1]
+        try:
+            is_auto_assigned = int(address, 16)
+        except ValueError:
+            pass
+
+        if is_auto_assigned:
+            return repr(self.state)
+        else:
+            return str(uri(self))
+
     def __id__(self):
         return hex_id(self.state)
 
     def __json__(self):
         return to_json(uri(self))
 
-    def __ns__(self, cxt):
-        deanonymize(self.state, cxt)
-
-    def __repr__(self):
-        return f"Ref({self.state}, {uri(self)})"
+    def __ns__(self, cxt, name_hint):
+        deanonymize(self.state, cxt, name_hint + '_' + str(uri(self))[1:].replace('/', '_'))
